package geniusweb.javavenv;

import java.io.IOException;
import java.util.logging.Level;

import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.Action;
import geniusweb.connection.ConnectionEnd;
import geniusweb.inform.Inform;
import geniusweb.party.Capabilities;
import geniusweb.party.Party;
import tudelft.utilities.logging.Reporter;

/**
 * Proxy to an instance of a party on the {@link PythonVenv}.
 */
public class PartyProxy implements Party {

	private final Pipe partypipe;
	private Capabilities capabilities;
	private String description;
	private ConnectionEnd<Inform, Action> connection;
	private final ObjectMapper jackson = new ObjectMapper();
	private Reporter logger;

	/**
	 * 
	 * @param partypipe the {@link Pipe} to the python party program
	 * @param caps      the party's capabilities (fetched from the program)
	 * @param desc      the party's description (fetched from the program)
	 * @param logger    the logger to report issues
	 * @throws IOException if we can not start up the process
	 */
	public PartyProxy(Pipe partypipe, Capabilities caps, String desc,
			Reporter logger) throws IOException {
		this.partypipe = partypipe;
		this.capabilities = caps;
		this.description = desc;
		this.logger = logger;

		partypipe.addListener(action -> partyActs(action));
		partypipe.run();
	}

	@Override
	public void connect(ConnectionEnd<Inform, Action> connection) {
		this.connection = connection;
		connection.addListener(info -> send(info));
	}

	/**
	 * @param info the {@link Inform} to send to the party
	 * @throws IOException
	 */
	private void send(Inform info) {
		try {
			partypipe.send(jackson.writeValueAsString(info));
		} catch (IOException e) {
			logger.log(Level.WARNING, e.getMessage(), e);
			// Parties can not do anything with errors.
		}
	}

	@Override
	public void disconnect() {
		// TODO Auto-generated method stub

	}

	@Override
	public Capabilities getCapabilities() {
		return capabilities;
	}

	@Override
	public String getDescription() {
		return description;
	}

	@Override
	public void terminate() {
		partypipe.terminate();
	}

	/**
	 * Called when the party sends us an action. We forward the action as if we
	 * did the action.
	 * 
	 * @param action the action, as string. If null, this indicates the pipe to
	 *               the party broke.
	 */
	private void partyActs(String action) {
		if (connection == null) {
			logger.log(Level.WARNING,
					"party is acting but proxy is not yet connected. Action "
							+ action + " is lost");
			return;
		}
		if (action == null) {
			try {
				connection.send(null);
			} catch (IOException e) {
				// FIXME is this to be expected? When???
				logger.log(Level.WARNING, "Error handling null action", e);
			}
		} else {
			try {
				connection.send(jackson.readValue(action, Action.class));
			} catch (IOException e) {
				logger.log(Level.WARNING, "Error sending action", e);
			}
		}
		// Check pipe status to our client.
		Exception err = partypipe.getError();
		if (err != null) {
			logger.log(Level.WARNING, "pipe contains error:", err);
		}

	}
}
